@@ -393,7 +393,7 @@ struct dwflpp
   template<typename T>
   void iterate_over_callees (Dwarf_Die *begin_die,
                              const std::string& sym,
-                             long recursion_depth,
+                             int64_t recursion_depth,
                              T *data,
                              void (* callback)(base_func_info&,
                                                base_func_info&,
@@ -613,8 +613,8 @@ private:
                                                  Dwarf_Die *vardie,
                                                  Dwarf_Attribute *fb_attr_mem);
 
-  std::string die_location_as_string(Dwarf_Addr, Dwarf_Die*);
-  std::string die_location_as_function_string(Dwarf_Addr, Dwarf_Die*);
+  std::string die_location_as_string(Dwarf_Die*);
+  std::string pc_location_as_function_string(Dwarf_Addr);
   std::string pc_die_line_string(Dwarf_Addr, Dwarf_Die*);
 
   /* source file name, line and column info for pc in current cu. */
@@ -762,7 +762,7 @@ dwflpp::iterate_over_labels<void>(Dwarf_Die *begin_die,
 template<> void
 dwflpp::iterate_over_callees<void>(Dwarf_Die *begin_die,
                                    const std::string& sym,
-                                   long recursion_depth,
+                                   int64_t recursion_depth,
                                    void *data,
                                    void (* callback)(base_func_info&,
                                                      base_func_info&,
