@@ -1,5 +1,5 @@
 // systemtap debuginfo rpm finder
-// Copyright (C) 2009-2011 Red Hat Inc.
+// Copyright (C) 2009-2014 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -75,6 +75,10 @@ missing_rpm_enlist (systemtap_session& sess, const char *filename, const char *r
       rpm_init_done = 1;
     }
 
+  /* If we've seen this combo before, don't check again... */
+  if (!sess.rpms_checked.insert(string(filename) + rpm_type).second)
+    return 0;
+
   ts = rpmtsCreate(); 
 
   mi = rpmtsInitIterator(ts, RPMTAG_BASENAMES, filename, 0);
@@ -197,10 +201,10 @@ missing_rpm_enlist (systemtap_session& sess, const char *filename, const char *r
 }
 #endif	/* HAVE_LIBRPM */
 
+#ifdef HAVE_LIBRPM
 void
 missing_rpm_list_print (systemtap_session &sess, const char* rpm_type)
 {
-#ifdef HAVE_LIBRPM
   if (sess.rpms_to_install.size() > 0 && ! sess.suppress_warnings) {
 
     if(strcmp(rpm_type,"-devel")==0)
@@ -221,26 +225,39 @@ missing_rpm_list_print (systemtap_session &sess, const char* rpm_type)
     }
     cerr << endl;
   }
-#endif
 }
+#else
+void
+missing_rpm_list_print (systemtap_session &, const char *)
+{
+}
+#endif
 
+#ifdef HAVE_LIBRPM
 int
 find_debug_rpms (systemtap_session &sess, const char * filename)
 {
-#ifdef HAVE_LIBRPM
   const char *rpm_type = "-debuginfo";
   return missing_rpm_enlist(sess, filename, rpm_type);
-#else
-  return 0;
-#endif
 }
+#else
+int
+find_debug_rpms (systemtap_session &, const char *)
+{
+  return 0;
+}
+#endif
 
+
+#ifdef HAVE_LIBRPM
 int find_devel_rpms(systemtap_session &sess, const char * filename)
 {
-#ifdef HAVE_LIBRPM
   const char *rpm_type = "-devel";
   return missing_rpm_enlist(sess, filename, rpm_type);
-#else
-  return 0;
-#endif
 }
+#else
+int find_devel_rpms(systemtap_session &, const char *)
+{
+  return 0;
+}
+#endif
