@@ -247,10 +247,12 @@ static void _stp_mem_debug_validate(void *addr)
  */
 
 #ifdef STP_MAXMEMORY
-#ifndef STAPCONF_GRSECURITY
-#define _STP_MODULE_CORE_SIZE (THIS_MODULE->core_size)
-#else
+#ifdef STAPCONF_MODULE_LAYOUT
+#define _STP_MODULE_CORE_SIZE (THIS_MODULE->core_layout.size)
+#elif defined(STAPCONF_GRSECURITY)
 #define _STP_MODULE_CORE_SIZE (THIS_MODULE->core_size_rw)
+#else
+#define _STP_MODULE_CORE_SIZE (THIS_MODULE->core_size)
 #endif
 #endif
 
