@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Map Runtime Functions
- * Copyright (C) 2012 Red Hat Inc.
+ * Copyright (C) 2012-2016 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -11,38 +11,10 @@
 #ifndef _LINUX_MAP_RUNTIME_H_
 #define _LINUX_MAP_RUNTIME_H_
 
-/* Include map spinlocks only on demand.  Otherwise, assume that
-   caller does the right thing. */
-#ifdef NEED_MAP_LOCKS
-
-#define MAP_LOCK(m)	spin_lock(&(m)->lock)
-#define MAP_UNLOCK(m)	spin_unlock(&(m)->lock)
-#define MAP_TRYLOCK(m)	spin_trylock(&(m)->lock)
-
-#define MAP_GET_CPU()	get_cpu()
-#define MAP_PUT_CPU()	put_cpu()
-#else  /* !NEED_MAP_LOCKS */
-
-#define MAP_LOCK(m)	do {} while (0)
-#define MAP_UNLOCK(m)	do {} while (0)
-#define MAP_TRYLOCK(m)	1
-
 /* get/put_cpu wrappers.  Unnecessary if caller is already atomic. */
 #define MAP_GET_CPU()	smp_processor_id()
 #define MAP_PUT_CPU()	do {} while (0)
 
-#endif
-
-
-static int _stp_map_initialize_lock(MAP m)
-{
-#ifdef NEED_MAP_LOCKS
-	spin_lock_init(&m->lock);
-#endif
-	return 0;
-}
-
-#define _stp_map_destroy_lock(m)	do {} while (0)
 
 struct pmap {
 	MAP agg;	/* aggregation map */
@@ -88,7 +60,6 @@ static void _stp_map_del(MAP map)
 	if (map->node_mem)
 		_stp_vfree(map->node_mem);
 
-	_stp_map_destroy_lock(map);
 	_stp_vfree(map);
 }
 
@@ -121,13 +92,16 @@ _stp_map_vzalloc(size_t size, int cpu)
 
 
 static int
-_stp_map_init(MAP m, unsigned max_entries, int wrap, int node_size, int cpu)
+_stp_map_init(MAP m, unsigned max_entries, unsigned hash_table_mask,
+              int wrap, int node_size, int cpu)
 {
 	unsigned i;
 
 	INIT_MLIST_HEAD(&m->pool);
 	INIT_MLIST_HEAD(&m->head);
-	for (i = 0; i < HASH_TABLE_SIZE; i++)
+
+        m->hash_table_mask = hash_table_mask;
+	for (i = 0; i <= hash_table_mask; i++)
 		INIT_MHLIST_HEAD(&m->hashes[i]);
 
 	m->maxnum = max_entries;
@@ -145,9 +119,6 @@ _stp_map_init(MAP m, unsigned max_entries, int wrap, int node_size, int cpu)
 		INIT_MHLIST_NODE(&node->hnode);
 	}
 
-	if (_stp_map_initialize_lock(m) != 0)
-		return -1;
-
 	return 0;
 }
 
@@ -165,12 +136,14 @@ static MAP
 _stp_map_new(unsigned max_entries, int wrap, int node_size, int cpu)
 {
 	MAP m;
-
-	m = _stp_map_vzalloc(sizeof(struct map_root), cpu);
+        unsigned hash_table_mask = HASHTABLESIZE(max_entries)-1; /* usable as bitmask */
+	m = _stp_map_vzalloc(sizeof(struct map_root) +
+                             sizeof(struct mhlist_head) * (hash_table_mask+1),
+                             cpu);
 	if (m == NULL)
 		return NULL;
 
-	if (_stp_map_init(m, max_entries, wrap, node_size, cpu)) {
+	if (_stp_map_init(m, max_entries, hash_table_mask, wrap, node_size, cpu)) {
 		_stp_map_del(m);
 		return NULL;
 	}
