@@ -112,12 +112,26 @@
 /* If the system has asm/syscall.h, use defines from it. */
 #include <asm/syscall.h>
 
+#if defined(__arm__)
+/* The syscall_get_nr() function on 3.17.1-302.fc21.armv7hl always
+ * returns 0 (since it was designed to be used with ftrace syscall
+ * tracing, not called from any context). So, let's use our function
+ * instead. */
+static inline long
+_stp_syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
+{
+	return regs->ARM_r7;
+}
+#else
+#define _stp_syscall_get_nr syscall_get_nr
+#endif
+
 #else  /* !STAPCONF_ASM_SYSCALL_H */
 
 /* If the system doesn't have asm/syscall.h, use our defines. */
 #if defined(__i386__) || defined(__x86_64__)
 static inline long
-syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
+_stp_syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 {
 #if defined(STAPCONF_X86_UNIREGS)
 	return regs->orig_ax;
@@ -131,7 +145,7 @@ syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 
 #if defined(__powerpc__)
 static inline long
-syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
+_stp_syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 {
 	return regs->gpr[0];
 }
@@ -139,7 +153,7 @@ syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 
 #if defined(__ia64__)
 static inline long
-syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
+_stp_syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 {
 	if ((long)regs->cr_ifs < 0) /* Not a syscall */
 		return -1;
@@ -155,16 +169,24 @@ syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 
 #if defined(__s390__) || defined(__s390x__)
 static inline long
-syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
+_stp_syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 {
 	// might need to be 'orig_gpr2'
 	return regs->gprs[2];
 }
 #endif
 
+#if defined(__aarch64__)
+static inline long
+_stp_syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
+{
+	return regs->syscallno;
+}
+#endif
+
 #if defined(__arm__)
 static inline long
-syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
+_stp_syscall_get_nr(struct task_struct *task, struct pt_regs *regs)
 {
 	return regs->ARM_r7;
 }
