@@ -1,7 +1,7 @@
-/* -*- linux-c -*- 
+/* -*- linux-c -*-
  * symbols.c - stp symbol and module functions
  *
- * Copyright (C) Red Hat Inc, 2006-2012
+ * Copyright (C) Red Hat Inc, 2006-2015
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -121,22 +121,55 @@ static unsigned _stp_module_nsections (struct module_sect_attrs *attrs)
 static int _stp_module_notifier (struct notifier_block * nb,
                                  unsigned long val, void *data)
 {
+        struct module *mod = data;
+        struct module_sect_attrs *attrs;
+        unsigned i, nsections;
+
+        (void) attrs;
+        (void) i;
+        (void) nsections;
+
+        if (!mod) { // so as to avoid null pointer checks later
+                WARN_ON (!mod);
+                return NOTIFY_DONE;
+        }
+
+        dbug_sym(1, "module notify %lu %s attrs %p\n",
+                 val, mod->name, mod->sect_attrs);
+
         /* Prior to 2.6.11, struct module contained a module_sections
            attribute vector rather than module_sect_attrs.  Prior to
            2.6.19, module_sect_attrs lacked a number-of-sections
-           field.  Past 3.8, MODULE_STATE_COMING is sent too early to 
+           field.  Past 3.8, MODULE_STATE_COMING is sent too early to
            let us probe module init functions.
 
            Without CONFIG_KALLSYMS, we don't get any of the
            related fields at all in struct module.  XXX: autoconf for
            that directly? */
 
-#if defined(CONFIG_KALLSYMS) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
-        struct module *mod = data;
-        struct module_sect_attrs *attrs;
-        unsigned i, nsections;
-        WARN_ON (!mod);
-
+#if defined(CONFIG_KALLSYMS)
+	// After kernel commit 4982223e51, module notifiers are being
+	// called too early to get module section info. So, we have to
+	// switch to using symbol+offset probing for modules.
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
+	// The module refresh code (in systemtap_module_refresh)
+	// assumes the 1st call is on module load and the 2nd is on
+	// module unload. So, we can't call systemtap_module_refresh()
+	// twice for module load (once for MODULE_STATE_COMING and
+	// once for MODULE_STATE_LIVE). In the MODULE_STATE_COMING
+	// state, the module's init function hasn't fired yet and we
+	// can register symbol+offset probes. In the MODULE_STATE_LIVE
+	// state, the module's init function has already been run (and
+	// the init section has been discarded). So, we'll ignore
+	// MODULE_STATE_LIVE.
+        if (val == MODULE_STATE_COMING) {
+		/* Verify build-id. */
+		_stp_kmodule_check (mod->name);
+        }
+        else if (val != MODULE_STATE_GOING) {
+		return NOTIFY_DONE;
+        }
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
         if (val == MODULE_STATE_COMING ||
             val == MODULE_STATE_LIVE) {
                 /* A module is arriving or has arrived.  Register all
@@ -145,6 +178,7 @@ static int _stp_module_notifier (struct notifier_block * nb,
                    did the fishie go? */
 
                 attrs = mod->sect_attrs;
+                dbug_sym(1, "module_sect_attrs: %p\n", attrs);
                 if (attrs == NULL) // until add_sect_attrs(), may be zero
                         return NOTIFY_DONE; // remain ignorant
 
@@ -153,7 +187,7 @@ static int _stp_module_notifier (struct notifier_block * nb,
                         int init_p = (strstr(attrs->attrs[i].name, "init.") != NULL);
                         int init_gone_p = (val == MODULE_STATE_LIVE); // likely already unloaded
 
-                        _stp_kmodule_update_address(mod->name, 
+                        _stp_kmodule_update_address(mod->name,
                                                     attrs->attrs[i].name,
                                                     ((init_p && init_gone_p) ? 0 : attrs->attrs[i].address));
                 }
@@ -161,11 +195,14 @@ static int _stp_module_notifier (struct notifier_block * nb,
                 /* Verify build-id. */
                 if (_stp_kmodule_check (mod->name))
                    _stp_kmodule_update_address(mod->name, NULL, 0); /* Pretend it was never here. */
-        }        
+        }
         else if (val == MODULE_STATE_GOING) {
                 /* Unregister all sections. */
                 _stp_kmodule_update_address(mod->name, NULL, 0);
         }
+	else
+		return NOTIFY_DONE;
+#endif
 
         /* Give the probes a chance to update themselves. */
         /* Proper kprobes support for this appears to be relatively
@@ -211,14 +248,27 @@ static int _stp_module_update_self (void)
 			found_eh_frame = true;
 		}
 		else if (!strcmp(".symtab", attr->name)) {
-			_stp_module_self.sections[0].static_addr = attr->address;
+#ifdef STAPCONF_MOD_KALLSYMS
+			struct mod_kallsyms *kallsyms = rcu_dereference_sched(mod->kallsyms);
+			if (attr->address == (unsigned long) kallsyms->symtab)
+				_stp_module_self.sections[0].size =
+					kallsyms->num_symtab * sizeof(kallsyms->symtab[0]);
+#else
 			if (attr->address == (unsigned long) mod->symtab)
 				_stp_module_self.sections[0].size =
 					mod->num_symtab * sizeof(mod->symtab[0]);
+#endif
+			_stp_module_self.sections[0].static_addr = attr->address;
 		}
 		else if (!strcmp(".text", attr->name)) {
 			_stp_module_self.sections[1].static_addr = attr->address;
+#ifdef STAPCONF_MODULE_LAYOUT
+			_stp_module_self.sections[1].size = mod->core_layout.text_size;
+#elif defined(STAPCONF_GRSECURITY)
+                        _stp_module_self.sections[1].size = mod->core_size_rx;
+#else
 			_stp_module_self.sections[1].size = mod->core_text_size;
+#endif
 		}
 	}
 
