@@ -74,9 +74,7 @@ arrayindex::arrayindex ():
 {
 }
 
-
-functioncall::functioncall ():
-  referent (0)
+functioncall::functioncall ()
 {
 }
 
@@ -124,29 +122,32 @@ probe_point::from_globby_comp(const std::string& comp)
 unsigned probe::last_probeidx = 0;
 
 probe::probe ():
-  body (0), base (0), tok (0), systemtap_v_conditional (0), privileged (false)
+  body (0), base (0), tok (0), systemtap_v_conditional (0), privileged (false),
+  id (last_probeidx ++)
 {
-  this->name = string ("probe_") + lex_cast(last_probeidx ++);
 }
 
 
 // Copy constructor, but with overriding probe-point.  To be used when
 // mapping script-level probe points to another one, early during pass
 // 2.  There should be no symbol resolution done yet.
-probe::probe(probe* p, probe_point* l)
+probe::probe(probe* p, probe_point* l):
+  locations (1, l), body (deep_copy_visitor::deep_copy (p->body)),
+  base (p), tok (p->tok), systemtap_v_conditional (p->systemtap_v_conditional),
+  privileged (p->privileged), id (last_probeidx ++)
 {
-  this->base = p;
-  this->name = string ("probe_") + lex_cast(last_probeidx ++);
-  this->tok = p->tok;
-  this->locations.push_back(l);
-  this->body = deep_copy_visitor::deep_copy(p->body);
-  this->privileged = p->privileged;
-  this->systemtap_v_conditional = p->systemtap_v_conditional;
   assert (p->locals.size() == 0);
   assert (p->unused_locals.size() == 0);
 }
 
 
+string
+probe::name () const
+{
+  return string ("probe_") + lex_cast(id);
+}
+
+
 probe_point::component::component ():
   arg (0), from_glob(false), tok(0)
 {
@@ -211,7 +212,7 @@ vardecl::compatible_arity (int a)
 
 
 functiondecl::functiondecl ():
-  body (0), synthetic (false), mangle_oldstyle (false)
+  body (0), synthetic (false), mangle_oldstyle (false), priority(1)
 {
 }
 
@@ -222,7 +223,7 @@ functiondecl::join (systemtap_session& s)
     throw SEMANTIC_ERROR (_("internal error, joining a non-synthetic function"), tok);
   if (!s.functions.insert (make_pair (name, this)).second)
     throw SEMANTIC_ERROR (_F("synthetic function '%s' conflicts with an existing function",
-                             name.c_str()), tok);
+                             name.to_string().c_str()), tok);
   tok->location.file->functions.push_back (this);
 }
 
@@ -241,12 +242,6 @@ literal_string::literal_string (interned_string v)
   type = pe_string;
 }
 
-literal_string::literal_string (const string& v)
-{
-  value = v;
-  type = pe_string;
-}
-
 
 ostream&
 operator << (ostream& o, const exp_type& e)
@@ -269,6 +264,7 @@ target_symbol::assert_no_components(const std::string& tapset, bool pretty_ok)
   if (components.empty())
     return;
 
+  const string& name = this->name;
   switch (components[0].type)
     {
     case comp_literal_array_index:
@@ -530,7 +526,7 @@ void perf_op::print (ostream& o) const
 
 void vardecl::print (ostream& o) const
 {
-  o << name;
+  o << unmangled_name;
   if(wrap)
     o << "%";
   if (maxsize > 0)
@@ -547,7 +543,7 @@ void vardecl::print (ostream& o) const
 
 void vardecl::printsig (ostream& o) const
 {
-  o << name;
+  o << unmangled_name;
   if(wrap)
      o << "%";
   if (maxsize > 0)
@@ -565,7 +561,7 @@ void vardecl::printsig (ostream& o) const
 
 void functiondecl::print (ostream& o) const
 {
-  o << "function " << name << " (";
+  o << "function " << unmangled_name << " (";
   for (unsigned i=0; i<formal_args.size(); i++)
     o << (i>0 ? ", " : "") << *formal_args[i];
   o << ")" << endl;
@@ -575,7 +571,7 @@ void functiondecl::print (ostream& o) const
 
 void functiondecl::printsig (ostream& o) const
 {
-  o << name << ":" << type << " (";
+  o << unmangled_name << ":" << type << " (";
   for (unsigned i=0; i<formal_args.size(); i++)
     o << (i>0 ? ", " : "")
       << *formal_args[i]
@@ -666,8 +662,17 @@ print_format*
 print_format::create(const token *t, const char *n)
 {
   bool stream, format, delim, newline, _char;
-  if (n == NULL) n = t->content.c_str();
-  const char *o = n;
+  interned_string type;
+  string str_type;
+
+  if (n == NULL)
+    {
+      type = t->content;
+      str_type = type;
+      n = str_type.c_str();
+    }
+  else
+    type = n;
 
   stream = true;
   format = delim = newline = _char = false;
@@ -710,7 +715,7 @@ print_format::create(const token *t, const char *n)
 	return NULL;
     }
 
-  print_format *pf = new print_format(stream, format, delim, newline, _char, o);
+  print_format *pf = new print_format(stream, format, delim, newline, _char, type);
   pf->tok = t;
   return pf;
 }
@@ -821,14 +826,14 @@ print_format::components_to_string(vector<format_component> const & components)
 }
 
 vector<print_format::format_component>
-print_format::string_to_components(interned_string str)
+print_format::string_to_components(string const & str)
 {
   format_component curr;
   vector<format_component> res;
 
   curr.clear();
 
-  interned_string::const_iterator i = str.begin();
+  string::const_iterator i = str.begin();
   string literal_str;
   
   while (i != str.end())
@@ -1056,7 +1061,7 @@ void print_format::print (ostream& o) const
   if (print_with_format)
     o << lex_cast_qstring (raw_components);
   if (print_with_delim)
-    o << lex_cast_qstring (delimiter.literal_string);
+    o << lex_cast_qstring (delimiter);
   if (hist)
     hist->print(o);
   for (vector<expression*>::const_iterator i = args.begin();
@@ -1146,7 +1151,7 @@ void block::print (ostream& o) const
 {
   o << "{" << endl;
   for (unsigned i=0; i<statements.size(); i++)
-    o << *statements [i] << endl;
+    o << *statements [i] << ";" << endl;
   o << "}";
 }
 
@@ -2296,24 +2301,28 @@ functioncall_traversing_visitor::visit_functioncall (functioncall* e)
 void
 functioncall_traversing_visitor::enter_functioncall (functioncall* e)
 {
-  // prevent infinite recursion
-  if (nested.find (e->referent) == nested.end ())
+  for (unsigned i = 0; i < e->referents.size(); i++)
     {
-      if (seen.find(e->referent) == seen.end())
-        seen.insert (e->referent);
-      nested.insert (e->referent);
-      // recurse
-      functiondecl* last_current_function = current_function;
-      current_function = e->referent;
-      e->referent->body->visit (this);
-      current_function = last_current_function;
-      nested.erase (e->referent);
+      functiondecl* referent = e->referents[i];
+      // prevent infinite recursion
+      if (nested.find (referent) == nested.end ())
+        {
+          if (seen.find(referent) == seen.end())
+            seen.insert (referent);
+          nested.insert (referent);
+          // recurse
+          functiondecl* last_current_function = current_function;
+          current_function = referent;
+          referent->body->visit (this);
+          current_function = last_current_function;
+          nested.erase (referent);
+        }
+      else { this->note_recursive_functioncall(e); }
     }
-  else { this->note_recursive_functioncall(e); }
 }
 
 void
-functioncall_traversing_visitor::note_recursive_functioncall (functioncall* e)
+functioncall_traversing_visitor::note_recursive_functioncall (functioncall*)
 {
 }
 
@@ -2394,6 +2403,25 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
 {
   assert (current_function); // only they get embedded code
 
+  /* We need to lock globals that are accessed through embedded C code */
+  for (unsigned i = 0; i < session.globals.size(); i++)
+    {
+      vardecl* v = session.globals[i];
+      string name = v->unmangled_name;
+      if (s->code.find("/* pragma:read:" + name + " */") != string::npos)
+        {
+          if (v->type == pe_stats)
+            throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), s->tok);
+          read.insert(v);
+        }
+      if (s->code.find("/* pragma:write:" + name + " */") != string::npos)
+        {
+          if (v->type == pe_stats)
+            throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), s->tok);
+          written.insert(v);
+        }
+    }
+
   // Don't allow embedded C functions in unprivileged mode unless
   // they are tagged with /* unprivileged */ or /* myproc-unprivileged */
   // or we're in a usermode runtime.
@@ -2436,6 +2464,25 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
 void
 varuse_collecting_visitor::visit_embedded_expr (embedded_expr *e)
 {
+  /* We need to lock globals that are accessed through embedded C code */
+  for (unsigned i = 0; i < session.globals.size(); i++)
+    {
+      vardecl* v = session.globals[i];
+      string name = v->unmangled_name;
+      if (e->code.find("/* pragma:read:" + name + " */") != string::npos)
+        {
+          if (v->type == pe_stats)
+            throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), e->tok);
+          read.insert(v);
+        }
+      if (e->code.find("/* pragma:write:" + name + " */") != string::npos)
+        {
+          if (v->type == pe_stats)
+            throw SEMANTIC_ERROR(_("Aggregates not available in embedded-C"), e->tok);
+          written.insert(v);
+        }
+    }
+
   // Don't allow embedded C expressions in unprivileged mode unless
   // they are tagged with /* unprivileged */ or /* myproc-unprivileged */
   // or we're in a usermode runtime.
@@ -3607,7 +3654,7 @@ void
 deep_copy_visitor::visit_functioncall (functioncall* e)
 {
   functioncall* n = new functioncall(*e);
-  n->referent = NULL; // don't copy!
+  n->referents.clear(); // don't copy!
   update_visitor::visit_functioncall(n);
 }
 
