@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2014 Red Hat Inc.
+// Copyright (C) 2005-2015 Red Hat Inc.
 // Copyright (C) 2006 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -24,6 +24,7 @@ extern "C" {
 }
 
 #include "util.h"
+#include "stringtable.h"
 
 #if defined(HAVE_TR1_MEMORY)
 #include <tr1/memory>
@@ -35,6 +36,7 @@ using boost::shared_ptr;
 #error "No shared_ptr implementation found; get boost or modern g++"
 #endif
 
+
 struct token; // parse.h
 struct systemtap_session; // session.h
 
@@ -81,11 +83,12 @@ struct semantic_error: public std::runtime_error
       return errsrc + (chain ? "|" + chain->errsrc_chain() : "");
     }
 
-  void set_chain(const semantic_error& new_chain)
+  semantic_error& set_chain(const semantic_error& new_chain)
     {
       if (chain)
         delete chain;
       chain = new semantic_error(new_chain);
+      return *this;
     }
 
   const semantic_error* get_chain(void) const
@@ -165,8 +168,8 @@ struct literal: public expression
 
 struct literal_string: public literal
 {
-  std::string value;
-  literal_string (const std::string& v);
+  interned_string value;
+  literal_string (interned_string v);
   void print (std::ostream& o) const;
   void visit (visitor* u);
 };
@@ -184,7 +187,7 @@ struct literal_number: public literal
 
 struct embedded_expr: public expression
 {
-  std::string code;
+  interned_string code;
   void print (std::ostream& o) const;
   void visit (visitor* u);
 };
@@ -193,7 +196,7 @@ struct embedded_expr: public expression
 struct binary_expression: public expression
 {
   expression* left;
-  std::string op;
+  interned_string op;
   expression* right;
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -202,7 +205,7 @@ struct binary_expression: public expression
 
 struct unary_expression: public expression
 {
-  std::string op;
+  interned_string op;
   expression* operand;
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -245,7 +248,7 @@ struct array_in: public expression
 struct regex_query: public expression
 {
   expression* left;
-  std::string op;
+  interned_string op;
   literal_string* right;
   void visit (visitor* u);
   void print (std::ostream& o) const;
@@ -300,7 +303,7 @@ classify_indexable(indexable* ix,
 struct vardecl;
 struct symbol: public indexable
 {
-  std::string name;
+  interned_string name;
   vardecl *referent;
   symbol ();
   void print (std::ostream& o) const;
@@ -342,7 +345,7 @@ struct target_symbol: public expression
       void print (std::ostream& o) const;
     };
 
-  std::string name;
+  interned_string name;
   bool addressof;
   std::vector<component> components;
   semantic_error* saved_conversion_error; // hand-made linked list
@@ -364,7 +367,7 @@ std::ostream& operator << (std::ostream& o, const target_symbol::component& c);
 struct cast_op: public target_symbol
 {
   expression *operand;
-  std::string type_name, module;
+  interned_string type_name, module;
   void print (std::ostream& o) const;
   void visit (visitor* u);
 };
@@ -380,7 +383,7 @@ struct autocast_op: public target_symbol
 
 struct atvar_op: public target_symbol
 {
-  std::string target_name, cu_name, module;
+  interned_string target_name, cu_name, module;
   virtual std::string sym_name ();
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -423,13 +426,12 @@ struct arrayindex: public expression
 struct functiondecl;
 struct functioncall: public expression
 {
-  std::string function;
+  interned_string function;
   std::vector<expression*> args;
-  functiondecl *referent;
+  std::vector<functiondecl*> referents;
   functioncall ();
   void print (std::ostream& o) const;
   void visit (visitor* u);
-  std::string var_assigned_to_retval;
 };
 
 
@@ -483,14 +485,14 @@ struct print_format: public expression
 
   struct format_component
   {
-    unsigned long flags;
     unsigned base;
     unsigned width;
     unsigned precision;
-    width_type widthtype;
-    precision_type prectype;
-    conversion_type type;
-    std::string literal_string;
+    unsigned flags : 8;
+    width_type widthtype : 8;
+    precision_type prectype : 8;
+    conversion_type type : 8;
+    interned_string literal_string;
     bool is_empty() const
     {
       return flags == 0
@@ -501,7 +503,9 @@ struct print_format: public expression
     }
     void clear()
     {
+      base = 0;
       flags = 0;
+      width = 0;
       widthtype = width_unspecified;
       precision = 0;
       prectype = prec_unspecified;
@@ -515,7 +519,7 @@ struct print_format: public expression
 
   std::string raw_components;
   std::vector<format_component> components;
-  format_component delimiter;
+  interned_string delimiter;
   std::vector<expression*> args;
   hist_op *hist;
 
@@ -527,8 +531,8 @@ struct print_format: public expression
   void visit (visitor* u);
 
 private:
-  std::string print_format_type;
-  print_format(bool stream, bool format, bool delim, bool newline, bool _char, std::string type):
+  interned_string print_format_type;
+  print_format(bool stream, bool format, bool delim, bool newline, bool _char, interned_string type):
     print_to_stream(stream), print_with_format(format),
     print_with_delim(delim), print_with_newline(newline),
     print_char(_char), hist(NULL), print_format_type(type)
@@ -579,7 +583,8 @@ struct symboldecl // unique object per (possibly implicit)
 {
   const token* tok;
   const token* systemtap_v_conditional; //checking systemtap compatibility
-  std::string name;
+  interned_string name; // mangled name
+  interned_string unmangled_name;
   exp_type type;
   exp_type_ptr type_details;
   symboldecl ();
@@ -623,6 +628,7 @@ struct functiondecl: public symboldecl
   statement* body;
   bool synthetic;
   bool mangle_oldstyle;
+  int64_t priority;
   functiondecl ();
   void print (std::ostream& o) const;
   void printsig (std::ostream& o) const;
@@ -630,6 +636,14 @@ struct functiondecl: public symboldecl
   void join (systemtap_session& s); // for synthetic functions only
 };
 
+struct function_priority_order
+{
+  bool operator() (const functiondecl* f1, const functiondecl* f2)
+  {
+    return f1->priority < f2->priority;
+  }
+};
+
 
 // ------------------------------------------------------------------------
 
@@ -649,7 +663,7 @@ std::ostream& operator << (std::ostream& o, const statement& k);
 
 struct embeddedcode: public statement
 {
-  std::string code;
+  interned_string code;
   void print (std::ostream& o) const;
   void visit (visitor* u);
 };
@@ -779,10 +793,10 @@ struct stapfile
   std::vector<functiondecl*> functions;
   std::vector<vardecl*> globals;
   std::vector<embeddedcode*> embeds;
-  std::string file_contents;
+  interned_string file_contents;
   bool privileged;
   bool synthetic; // via parse_synthetic_*
-  stapfile (): file_contents (""),
+  stapfile ():
     privileged (false), synthetic (false) {}
   void print (std::ostream& o) const;
 };
@@ -792,16 +806,16 @@ struct probe_point
 {
   struct component // XXX: sort of a restricted functioncall
   {
-    std::string functor;
+    interned_string functor;
     literal* arg; // optional
+    bool from_glob;
     component ();
     const token* tok; // points to component's functor
-    component(std::string const & f, literal * a = NULL);
+    component(interned_string f, literal *a=NULL, bool from_glob=false);
   };
   std::vector<component*> components;
   bool optional;
   bool sufficient;
-  bool from_glob;
   bool well_formed; // used in derived_probe::script_location()
   expression* condition;
   void print (std::ostream& o, bool print_extras=true) const;
@@ -809,6 +823,7 @@ struct probe_point
   probe_point(const probe_point& pp);
   probe_point(std::vector<component*> const & comps);
   std::string str(bool print_extras=true) const;
+  bool from_globby_comp(const std::string& comp);
 };
 
 std::ostream& operator << (std::ostream& o, const probe_point& k);
@@ -816,6 +831,8 @@ std::ostream& operator << (std::ostream& o, const probe_point& k);
 
 struct probe
 {
+  static unsigned last_probeidx;
+
   std::vector<probe_point*> locations;
   statement* body;
   struct probe* base;
@@ -823,18 +840,24 @@ struct probe
   const token* systemtap_v_conditional; //checking systemtap compatibility
   std::vector<vardecl*> locals;
   std::vector<vardecl*> unused_locals;
-  static unsigned last_probeidx;
+  bool privileged;
+  unsigned id;
+
   probe ();
   probe (probe* p, probe_point *l);
   void print (std::ostream& o) const;
+  std::string name () const;
   virtual void printsig (std::ostream &o) const;
   virtual void collect_derivation_chain (std::vector<probe*> &probes_list) const;
   virtual void collect_derivation_pp_chain (std::vector<probe_point*> &) const;
   virtual const probe_alias *get_alias () const { return 0; }
   virtual probe_point *get_alias_loc () const { return 0; }
   virtual ~probe() {}
-  bool privileged;
-  std::string name;
+
+private:
+
+  probe (const probe&);
+  probe& operator = (const probe&);
 };
 
 struct probe_alias: public probe
@@ -901,6 +924,55 @@ struct visitor
 };
 
 
+// A NOP visitor doesn't do anything.  It's a useful base class if you need to
+// take action only for specific types, without even traversing the rest.
+struct nop_visitor: public visitor
+{
+  virtual ~nop_visitor () {}
+  virtual void visit_block (block *) {};
+  virtual void visit_try_block (try_block *) {};
+  virtual void visit_embeddedcode (embeddedcode *) {};
+  virtual void visit_null_statement (null_statement *) {};
+  virtual void visit_expr_statement (expr_statement *) {};
+  virtual void visit_if_statement (if_statement*) {};
+  virtual void visit_for_loop (for_loop*) {};
+  virtual void visit_foreach_loop (foreach_loop*) {};
+  virtual void visit_return_statement (return_statement*) {};
+  virtual void visit_delete_statement (delete_statement*) {};
+  virtual void visit_next_statement (next_statement*) {};
+  virtual void visit_break_statement (break_statement*) {};
+  virtual void visit_continue_statement (continue_statement*) {};
+  virtual void visit_literal_string (literal_string*) {};
+  virtual void visit_literal_number (literal_number*) {};
+  virtual void visit_embedded_expr (embedded_expr*) {};
+  virtual void visit_binary_expression (binary_expression*) {};
+  virtual void visit_unary_expression (unary_expression*) {};
+  virtual void visit_pre_crement (pre_crement*) {};
+  virtual void visit_post_crement (post_crement*) {};
+  virtual void visit_logical_or_expr (logical_or_expr*) {};
+  virtual void visit_logical_and_expr (logical_and_expr*) {};
+  virtual void visit_array_in (array_in*) {};
+  virtual void visit_regex_query (regex_query*) {};
+  virtual void visit_comparison (comparison*) {};
+  virtual void visit_concatenation (concatenation*) {};
+  virtual void visit_ternary_expression (ternary_expression*) {};
+  virtual void visit_assignment (assignment*) {};
+  virtual void visit_symbol (symbol*) {};
+  virtual void visit_target_symbol (target_symbol*) {};
+  virtual void visit_arrayindex (arrayindex*) {};
+  virtual void visit_functioncall (functioncall*) {};
+  virtual void visit_print_format (print_format*) {};
+  virtual void visit_stat_op (stat_op*) {};
+  virtual void visit_hist_op (hist_op*) {};
+  virtual void visit_cast_op (cast_op*) {};
+  virtual void visit_autocast_op (autocast_op*) {};
+  virtual void visit_atvar_op (atvar_op*) {};
+  virtual void visit_defined_op (defined_op*) {};
+  virtual void visit_entry_op (entry_op*) {};
+  virtual void visit_perf_op (perf_op*) {};
+};
+
+
 // A simple kind of visitor, which travels down to the leaves of the
 // statement/expression tree, up to but excluding following vardecls
 // and functioncalls.
