@@ -24,6 +24,7 @@
 #include <fstream>
 #include <cassert>
 #include <ext/stdio_filebuf.h>
+#include <algorithm>
 
 extern "C" {
 #include <elf.h>
@@ -536,6 +537,18 @@ const string cmdstr_quoted(const string& cmd)
 	return quoted_cmd;
 }
 
+const string
+detox_path(const string& str)
+{
+  ostringstream hash;
+  for (int i=0; i<int(str.length()); i++)
+    if (isalnum(str[i]))
+      hash << str[i];
+    else
+      hash << "_";
+  hash << "_";
+  return hash.str();
+}
 
 const string
 cmdstr_join(const vector<string>& cmds)
@@ -552,6 +565,21 @@ cmdstr_join(const vector<string>& cmds)
 }
 
 
+const string
+join(const vector<string>& vec, const string &delim)
+{
+  if (vec.empty())
+    throw runtime_error(_("join called with an empty vector!"));
+
+  stringstream join_str;
+  join_str << vec[0];
+  for (size_t i = 1; i < vec.size(); ++i)
+    join_str << delim << vec[i];
+
+  return join_str.str();
+}
+
+
 // signal-safe set of pids
 class spawned_pids_t {
   private:
@@ -877,6 +905,48 @@ stap_system_read(int verbose, const vector<string>& args, ostream& out)
 }
 
 
+std::pair<bool,int>
+stap_fork_read(int verbose, ostream& out)
+{
+  int pipefd[2];
+  if (pipe(pipefd) != 0)
+    return make_pair(false, -1);
+
+  fflush(stdout); cout.flush();
+
+  if (verbose > 1)
+    clog << _("Forking subprocess...") << endl;
+
+  pid_t child = fork();
+  PROBE1(stap, stap_system__fork, child);
+  // child < 0: fork failure
+  if (child < 0)
+    {
+      if (verbose > 1)
+        clog << _F("Fork error (%d): %s", child, strerror(errno)) << endl;
+      close(pipefd[0]);
+      close(pipefd[1]);
+      return make_pair(false, -1);
+    }
+  // child == 0: we're the child
+  else if (child == 0)
+    {
+      close(pipefd[0]);
+      fcntl(pipefd[1], F_SETFD, FD_CLOEXEC);
+      return make_pair(true, pipefd[1]);
+    }
+
+  // child > 0: we're the parent
+  spawned_pids.insert(child);
+
+  // read everything from the child
+  close(pipefd[1]);
+  stdio_filebuf<char> in(pipefd[0], ios_base::in);
+  out << &in;
+  return make_pair(false, stap_waitpid(verbose, child));
+}
+
+
 // Send a signal to our spawned commands
 int
 kill_stap_spawn(int sig)
